function mf604(command,p2)
%MF604 Hardware driver setup for the MF604 board.
%
%   This function creates the GUI for setting device driver parameters 
%   for the MF604 board.

%	Version 3.10  5-January-2001
%	Copyright (c) 1992-2001   HUMUSOFT s.r.o.

boardname=rttool('Load','mf604');
figh=findobj('Type','figure','Name',boardname);

if isempty(figh)

% Draw the GUI figure

figh=dialog('HandleVisibility','on','Visible','off','Name',boardname,'Tag','RTDRIVER_Window',...
            'Position',[100 80 325 250],'KeyPressFcn',...
            'mf604 @key');

% Define Address control

uicontrol('Style','text','Position',[20 218 70 20],'HorizontalAlignment','Left',...
          'String','Address :');
uicontrol('Style','edit','BackGroundColor',[1 1 1],'Position',[20 200 40 20],...
          'HorizontalAlignment','Left','CallBack','mf604 @addr;mf604 @cline;',...
          'Tag','Address_Field');
for i=1:4;
  uicontrol('Style','check','Position',[68+27*i 200 20 20],...
            'Max',2^(9-i),'CallBack','mf604 @addrb;mf604 @cline;','Tag','Address_Bits');
  uicontrol('Style','text','Position',[65+27*i 218 24 20],'String',...
            sprintf('A%1d',9-i));
end;

% Define Input format control

uicontrol('Style','text','Position',[50 162 90 20],'HorizontalAlignment','Left',...
          'String','Digital Input:');
uicontrol('Style','radio','Position',[50 145 45 20],'String','Bit','Min',-1,'Max',0,...
          'CallBack','rbutton;mf604 @cline;','Tag','Input_Format');
uicontrol('Style','radio','Position',[100 145 60 20],'String','Byte','Max',2,...
          'CallBack','rbutton;mf604 @cline;','Tag','Input_Format');
  
% Define Output format control

uicontrol('Style','text','Position',[190 162 90 20],'HorizontalAlignment','Left',...
          'String','Digital Output:');
uicontrol('Style','radio','Position',[190 145 45 20],'String','Bit','Min',-1,'Max',0,...
          'CallBack','rbutton;mf604 @cline;','Tag','Output_Format');
uicontrol('Style','radio','Position',[240 145 60 20],'String','Byte','Max',3,...
          'CallBack','rbutton;mf604 @cline;','Tag','Output_Format');

% Define Gain control

uicontrol('Style','push','Position',[20 100 85 25],'String','Gains ...',...
          'CallBack','mf604 @gain','Tag','Gain_Button');

% Define IRC filters control

uicontrol('Style','push','Position',[120 100 85 25],'String','IRC filters ...',...
          'CallBack','mf604 @mf604irc','Tag','MF604Irc_Button');

% Define Counters control

uicontrol('Style','push','Position',[220 100 85 25],'String','Counters ...',...
          'CallBack','mf604 @mf604ctr','Tag','MF604Ctr_Button');

% Define Command line control

uicontrol('Style','text','Position',[10 75 100 16],'HorizontalAlignment','Left',...
          'String','Command line:');
uicontrol('Style','text','BackGroundColor',[1 1 1],'HorizontalAlignment','Left',...
          'Position',[10 45 305 30],'Tag','Command_Line');

% Define buttons

uicontrol('Style','push','Position',[22 10 56 25],'String','OK',...
          'CallBack','mf604 @ok');
uicontrol('Style','push','Position',[97 10 56 25],'String','Reset',...
          'CallBack','mf604 @default');
uicontrol('Style','push','Position',[172 10 56 25],'String','Save',...
          'CallBack','mf604 @cline @save');
uicontrol('Style','push','Position',[247 10 56 25],'String','Cancel',...
          'CallBack','close');
  
% Draw figure

set(findobj(figh,'Units','pixels'),'Units','normalized');
if nargin<2
  drvs=rttool('Read',0,'Drivers');
  drvf=0;
  for i=drvs
    if strcmp(rttool('Read',i,'Name'),'mf604')
      drvf=i;
      break;
    end;
  end;
  if drvf
    [addr,opt]=rttool('Read',drvf,'Address');
    mf604(addr,opt);
  else
    mf604 @default;
  end;
else
  mf604(command,p2);
end;
set(figh,'Visible','on');
drawnow;

else

% Bring the window to front if already open

  if nargin==0
    figure(figh);

% Set default parameters to GUI window

  elseif ~isstr(command)
    set(findobj(figh,'Tag','Address_Field'),'String',dec2hex(command));
    mf604 @addr;
    butt=findobj(figh,'Tag','Input_Format');
    bmax=get(butt,'Max');
    bmin=get(butt,'Min');
    bmax=[bmax{:}]-[bmin{:}];
    rbutton(butt(find(bmax==rem(p2(1),2)+1)));
    butt=findobj(figh,'Tag','Output_Format');
    bmax=get(butt,'Max');
    bmin=get(butt,'Min');
    bmax=[bmax{:}]-[bmin{:}];
    rbutton(butt(find(bmax==rem(p2(1),4)-rem(p2(1),2)+1)));
    gbv=zeros(4,8);
    gains=[p2(2:size(p2,2)) zeros(1,9-size(p2,2))];
    for i=1:8
      gbv(find([0:3]==gains(i)),i)=1;
    end;
    set(findobj(figh,'Tag','Gain_Button'),'UserData',gbv);
    ircv = round(2.5e6./[p2(10:min(13, size(p2,2))) ones(1,13-size(p2,2))]);
    set(findobj(figh,'Tag','MF604Irc_Button'), 'UserData', ircv);
    gbv=zeros(5,4);
    ctrmodes=[p2(14:size(p2,2)) zeros(1,17-size(p2,2))];
    for i=1:4
      gbv(find([0:4]==ctrmodes(i)),i)=1;
    end;
    set(findobj(figh,'Tag','MF604Ctr_Button'),'UserData',gbv);
    mf604 @cline;

% Call if Address bits changed

  elseif strcmp(command,'@addrb')
    addr = get(findobj(figh,'Tag','Address_Bits'), 'Value');
    addr = sum([addr{:} 512]);
    set(findobj(figh,'Tag','Address_Field'),'Value',addr,'String',dec2hex(addr));
   
% Call if Address field changed

  elseif strcmp(command,'@addr')
    a=hex2dec(get(findobj(figh,'Tag','Address_Field'),'String'));
    addrbh=findobj(figh,'Tag','Address_Bits');
    for i=addrbh'
      bmax=get(i,'Max');
      set(i,'Value',bmax*rem(fix(a/bmax),2));
    end;
    mf604 @addrb;

% Call if Gain pressed

  elseif strcmp(command,'@gain')
    gbv=get(findobj(figh,'Tag','Gain_Button'),'UserData');
    gbh=zeros(size(gbv));
    chfh=dialog('Visible','off',...
                'Name','Select Input Gains',...
                'Position',[122 156 280 98]);
    for i=1:8
      uicontrol('Style','text','Position',[70+15*i 81 9 12],...
                'String',rem(i,10));
      for j=1:4
        gbh(j,i)=uicontrol('Style','radio','Position',[67+15*i 80-18*j 15 15],...
                           'Value',gbv(j,i),'CallBack','rbutton',...
                           'Tag',sprintf('Gain_%d',i));
      end;
    end;

    gains=str2mat('05 V','5 V','010 V','10 V');
    for j=1:4
      uicontrol('Style','push','Position',[7 79-18*j 70 18],...
                'String',[gains(j,:) ' >>'],...
                'CallBack',sprintf('mf604(''@gainbutt'',%d)',j));
    end;
    uicontrol('Style','push','Position',[220 45 50 25],...
              'String','OK',...
              'CallBack','mf604 @gainok');
    uicontrol('Style','push','Position',[220 10 50 25],...
              'String','Cancel',...
              'CallBack','close');

    set(chfh,'UserData',gbh,'Visible','on');

% Call if Gain OK pressed

  elseif strcmp(command,'@gainok')
    gbh=get(gcf,'UserData');
    gbv=get(gbh,'Value');
    set(findobj(figh,'Tag','Gain_Button'), 'UserData', reshape([gbv{:}],size(gbh)));
    close;
    mf604 @cline;

% Call if Gain button pressed

  elseif strcmp(command,'@gainbutt')
    gbh=get(gcf,'UserData');
    for i=gbh(p2,:)
      rbutton(i);
    end;

% Call if IRC filters pressed

  elseif strcmp(command,'@mf604irc')
    ircv=get(findobj(figh,'Tag','MF604Irc_Button'),'UserData');
    chfh=dialog('Visible','off',...
                'Name','Select IRC filter frequencies',...
                'Position',[18 149 490 112]);
    for i=1:4
      uicontrol('Style','text','Position',[10 102-23*i 30 18],...
                'HorizontalAlignment','left','String',sprintf('IRC%1d',i));
      uicontrol('Style','slider','Position',[50 104-23*i 290 18],...
                'Min',0,'Max',255,'SliderStep',[1 10]/256,'Value',256-ircv(i),...
                'Tag','MF604Irc_Slider','UserData',i,'Callback',sprintf('mf604(''@mf604slider'',%d)',i));
      uicontrol('Style','edit','Position',[350 104-23*i 40 18],...
                'HorizontalAlignment','left','BackgroundColor','w',...
                'Tag','MF604Irc_Freq','UserData',i,'Callback','mf604 @mf604freq');
      uicontrol('Style','text','Position',[395 102-23*i 20 18],...
                'HorizontalAlignment','left','String','kHz');
      mf604('@mf604slider',i);
    end;

    uicontrol('Style','push','Position',[430 45 50 25],...
              'String','OK',...
              'CallBack','mf604 @mf604ircok');
    uicontrol('Style','push','Position',[430 10 50 25],...
              'String','Cancel',...
              'CallBack','close');

    set(chfh,'Visible','on');
    drawnow;

% Call if slider position changed

  elseif strcmp(command,'@mf604slider')
    sh = findobj(gcf,'Tag','MF604Irc_Slider','UserData',p2);
    eh = findobj(gcf,'Tag','MF604Irc_Freq','UserData',p2);
    v = 2.5e3/(256-round(get(sh,'Value')));
    roundord = 10^fix(log10(v)-4);
    v = roundord*round(v/roundord);
    set(eh,'String',sprintf('%g',v));

% Call if slider position changed

  elseif strcmp(command,'@mf604freq')
    eh = gcbo;
    idx = get(eh,'UserData');
    sh = findobj(gcf,'Tag','MF604Irc_Slider','UserData',idx);
    val = max(min(round(2.5e3/str2double(get(eh,'String'))),256),1);
    set(sh,'Value',256-val);
    mf604('@mf604slider',idx);

% Call if IRC OK pressed

  elseif strcmp(command,'@mf604ircok')
    ircv = zeros(1,4);
    for i=1:4
      sh = findobj(gcf,'Tag','MF604Irc_Slider','UserData',i);
      ircv(i) = 256-get(sh,'Value');
    end;
    set(findobj(figh,'Tag','MF604Irc_Button'), 'UserData', round(ircv));
    close;
    mf604 @cline;

% Call if Counters pressed

  elseif strcmp(command,'@mf604ctr')
    gbv=get(findobj(figh,'Tag','MF604Ctr_Button'),'UserData');
    gbh=zeros(size(gbv));
    chfh=dialog('Visible','off',...
                'Name','Select Counter Modes',...
                'Position',[133 142 258 125]);
    for i=1:4
      uicontrol('Style','text','Position',[103+15*i 110 9 12],...
                'String',rem(i,10));
      for j=1:5
        gbh(j,i)=uicontrol('Style','radio','Position',[100+15*i 98-18*j 15 15],...
                           'Value',gbv(j,i),'CallBack','rbutton',...
                           'Tag',sprintf('MF604Ctr_%d',i));
      end;
    end;

    ctrmodes=str2mat('Counter with reset','Counter without reset','Chained counter','Frequency generator','Delayed pulse');
    for j=1:5
      uicontrol('Style','text','Position',[10 97-18*j 103 18],...
                'String',ctrmodes(j,:),'HorizontalAlignment','left');
    end;
    uicontrol('Style','push','Position',[198 45 50 25],...
              'String','OK',...
              'CallBack','mf604 @mf604ctrok');
    uicontrol('Style','push','Position',[198 10 50 25],...
              'String','Cancel',...
              'CallBack','close');

    set(chfh,'UserData',gbh,'Visible','on');
    drawnow;

% Call if Counters OK pressed

  elseif strcmp(command,'@mf604ctrok')
    gbh=get(gcf,'UserData');
    gbv=get(gbh,'Value');
    set(findobj(figh,'Tag','MF604Ctr_Button'), 'UserData', reshape([gbv{:}],size(gbh)));
    close;
    mf604 @cline;


% Call if Command line to be updated

  elseif strcmp(command,'@cline')
    addr=get(findobj(figh,'Tag','Address_Field'),'Value');
    ioform = get([findobj(figh,'Tag','Input_Format')' findobj(figh,'Tag','Output_Format')'], 'Value');
    ioform = sum([ioform{:}]);
    og=[0:3]*get(findobj(figh,'Tag','Gain_Button'),'UserData');
    irc = round(2.5e6./get(findobj(figh,'Tag','MF604Irc_Button'),'UserData'));
    ctr = [0:4]*get(findobj(figh,'Tag','MF604Ctr_Button'),'UserData');
    ga = [irc ctr];
    cl=sprintf('rtload(''mf604'',%d,[%d',addr,ioform);
    cl=[cl sprintf(' %d',og)];
    cl=[cl sprintf(' %d',ga)];
    cl=[cl ']);'];
    set(findobj(figh,'Tag','Command_Line'),'String',cl);
    if nargin==2
      opt=str2num(cl(min(find(cl=='[')):max(find(cl==']'))));
      [f p]=uiputfile('mf604.mat','Save driver defaults');
      if f
        save([p f],'addr','opt');
      end;
    end;

% Call if OK pressed

  elseif strcmp(command,'@ok')
    try
      rteval_(get(findobj(figh,'Tag','Command_Line'),'String'));
      close;
    catch
      rterror(lasterr);
    end;

% Call if Reset pressed

  elseif strcmp(command,'@default')
    if exist('mf604.mat')==2
      load mf604;
    else
      [dummy,addr,opt]=rttool('Load','mf604');
    end;
    mf604(addr,opt);

% Call if key pressed

  elseif strcmp(command,'@key')
    ch=get(figh,'CurrentCharacter');
    if ch==27
      close;
    elseif ch==13
      mf604 @ok;
    elseif upper(ch)=='R'
      mf604 @revert;
    elseif upper(ch)=='S'
      mf604 @cline @save;
    elseif upper(ch)=='G'
      mf604 @gain;
    elseif upper(ch)=='A'
      mf604 @gating;
    end;

  end

end


