/* $Revision:  $ $Date:  $ */
/* adhsmf604.c - xPC Target, non-inlined S-function driver for A/D section of Humusoft MF604 board  */
/* Copyright (c) 1994-1999 by The MathWorks, Inc. All Rights Reserved. */

#define 	S_FUNCTION_LEVEL 	2
#undef 		S_FUNCTION_NAME
#define 	S_FUNCTION_NAME 	adhsmf604

#include 	<stddef.h>
#include 	<stdlib.h>

#include 	"simstruc.h" 

#ifdef 		MATLAB_MEX_FILE
#include 	"mex.h"
#else
#include 	<windows.h>
#include 	"io_xpcimport.h"
#endif


/* Input Arguments */
#define NUM_PARAMS             (5)
#define BASE_ADDRESS_ARG       (ssGetSFcnParam(S,0))
#define DEV_ARG                (ssGetSFcnParam(S,1))
#define CHANNEL_ARG            (ssGetSFcnParam(S,2))
#define RANGE_ARG              (ssGetSFcnParam(S,3))
#define SAMPLE_TIME_PARAM      (ssGetSFcnParam(S,4))

/* Convert S Function Parameters to Varibles */

#define BASE                   ((uint_T) mxGetPr(BASE_ADDRESS_ARG)[0])
#define SAMPLE_TIME            ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[0])
#define SAMPLE_OFFSET          ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[1])


static char_T msg[256];


/*====================*
 * S-function methods *
 *====================*/

static void mdlCheckParameters(SimStruct *S)
{
}

static void mdlInitializeSizes(SimStruct *S)
{
    uint_T i;

#ifndef MATLAB_MEX_FILE
#include "io_xpcimport.c"
#endif


    ssSetNumSFcnParams(S, NUM_PARAMS);
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S)) {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL) {
            return;
        }
    } else {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

	

    if (!ssSetNumInputPorts(S, 0)) return;

    if (!ssSetNumOutputPorts(S, mxGetNumberOfElements(CHANNEL_ARG))) return;

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        ssSetOutputPortWidth(S, i, 1);
    }

    ssSetNumSampleTimes(S, 1);

    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetSFcnParamNotTunable(S,0);
    ssSetSFcnParamNotTunable(S,1);
    ssSetSFcnParamNotTunable(S,2);
    ssSetSFcnParamNotTunable(S,3);
    ssSetSFcnParamNotTunable(S,4);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE | SS_OPTION_PLACE_ASAP);
	
}


 
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, SAMPLE_TIME);
    ssSetOffsetTime(S, 0, SAMPLE_OFFSET);
}

 

#define MDL_START
static void mdlStart(SimStruct *S)
{

#ifndef MATLAB_MEX_FILE


    uint_T i, channel;
    char devName[20];

    strcpy(devName,"Humusoft MF604");

    /* Check That The Hardware is functioning */

    rl32eInpW(BASE+6);              /* dummy read from A/D */
    if (!(rl32eInpB(BASE+8) & 0x80)) { /* if data ready, error */
        sprintf(msg,"%s (0x%x): test A/D conversion failed", devName, BASE);
        ssSetErrorStatus(S,msg);
        return;
    }

    rl32eOutpB(BASE+6,0x40);        /* try the conversion */
    for (i=0;i<0x10000;i++)
        if (!(rl32eInpB(BASE+8) & 0x80)) break;
    if (i==0x10000) {               /* if no conversion, error */
        sprintf(msg,"%s (0x%x): test A/D conversion failed", devName, BASE);
        ssSetErrorStatus(S,msg);
        return;
    }

    rl32eInpW(BASE+6);
    if (!(rl32eInpB(BASE+8) & 0x80)) { /* if data still ready, error */
        sprintf(msg,"%s (0x%x): test A/D conversion failed", devName, BASE);
        ssSetErrorStatus(S,msg);
        return;
    }

#endif
                 
}

static void mdlOutputs(SimStruct *S, int_T tid)
{

#ifndef MATLAB_MEX_FILE

    uint_T i, n, channel;
    uint_T baseAddr = BASE;
    real_T *y, range;
    short adParameters;
    real_T res;

	
    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        channel=(uint_T)mxGetPr(CHANNEL_ARG)[i]-1;
        y=ssGetOutputPortSignal(S,i);
        range=mxGetPr(RANGE_ARG)[i];

        adParameters = 0x40;
        adParameters |= (channel & 0x07);
        if (range<0) adParameters |= 0x08;      /* bipolar range */
        if (abs(range)>5) adParameters |= 0x10; /* 10 V range */

        rl32eOutpB(baseAddr+6,adParameters);    /* start conversion */
        while (rl32eInpB(baseAddr+8) & 0x80);   /* wait until data ready */

        res=(short)rl32eInpW(baseAddr+6);       /* read data */
        if (range>0) {                          /* unipolar input */
            y[0]=res*range/4096; 
        } else {                                /* bipolar input */
            y[0]=-res*range/2048; 

        }
    }

#endif
        	        
}

static void mdlTerminate(SimStruct *S)
{   
}


#ifdef MATLAB_MEX_FILE  /* Is this file being compiled as a MEX-file? */
#include "simulink.c"   /* Mex glue */
#else
#include "cg_sfun.h"    /* Code generation glue */
#endif


