/* $Revision:  $ $Date:  $ */
/* enchsmf604.c - xPC Target, non-inlined S-function driver for encoder section of Humusoft MF604 board  */
/* Copyright (c) 1994-1999 by The MathWorks, Inc. All Rights Reserved. */

#define 	S_FUNCTION_LEVEL 	2
#undef 		S_FUNCTION_NAME
#define 	S_FUNCTION_NAME 	enchsmf604

#include 	<stddef.h>
#include 	<stdlib.h>

#include 	"simstruc.h" 

#ifdef 		MATLAB_MEX_FILE
#include 	"mex.h"
#else
#include 	<windows.h>
#include 	"io_xpcimport.h"
#endif


/* Input Arguments */
#define NUM_PARAMS             (5)
#define BASE_ADDRESS_ARG       (ssGetSFcnParam(S,0))
#define DEV_ARG                (ssGetSFcnParam(S,1))
#define CHANNEL_ARG            (ssGetSFcnParam(S,2))
#define FILTER_ARG             (ssGetSFcnParam(S,3))
#define SAMPLE_TIME_PARAM      (ssGetSFcnParam(S,4))

/* Convert S Function Parameters to Varibles */

#define BASE                   ((uint_T) mxGetPr(BASE_ADDRESS_ARG)[0])
#define SAMPLE_TIME            ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[0])
#define SAMPLE_OFFSET          ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[1])

#define MAXIRCFILT 2500000


static char_T msg[256];


/*====================*
 * S-function methods *
 *====================*/

static void mdlCheckParameters(SimStruct *S)
{
}

static void mdlInitializeSizes(SimStruct *S)
{
    uint_T i;

#ifndef MATLAB_MEX_FILE
#include "io_xpcimport.c"
#endif


    ssSetNumSFcnParams(S, NUM_PARAMS);
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S)) {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL) {
            return;
        }
    } else {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

	

    if (!ssSetNumInputPorts(S, 0)) return;

    if (!ssSetNumOutputPorts(S, mxGetNumberOfElements(CHANNEL_ARG))) return;

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        ssSetOutputPortWidth(S, i, 1);
    }

    ssSetNumSampleTimes(S, 1);

    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetSFcnParamNotTunable(S,0);
    ssSetSFcnParamNotTunable(S,1);
    ssSetSFcnParamNotTunable(S,2);
    ssSetSFcnParamNotTunable(S,3);
    ssSetSFcnParamNotTunable(S,4);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE | SS_OPTION_PLACE_ASAP);
	
}


 
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, SAMPLE_TIME);
    ssSetOffsetTime(S, 0, SAMPLE_OFFSET);
}

 

#define MDL_START
static void mdlStart(SimStruct *S)
{

#ifndef MATLAB_MEX_FILE

    uint_T i, channel;
    char devName[20];

    strcpy(devName,"Humusoft MF604");

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        channel=(uint_T)mxGetPr(CHANNEL_ARG)[i]-1;

        rl32eOutpB(BASE+0x11+2*channel,0x01);    // reset BP
        rl32eOutpB(BASE+0x10+2*channel, min(255, (uint_T)(MAXIRCFILT/mxGetPr(FILTER_ARG)[i])-0.5));  // program filter frequency
        rl32eOutpB(BASE+0x11+2*channel,0x18);    // PR0 -> PSC
        rl32eOutpB(BASE+0x11+2*channel,0x01);    // reset BP
        rl32eOutpB(BASE+0x10+2*channel,0x00);    // reset PR
        rl32eOutpB(BASE+0x10+2*channel,0x00);
        rl32eOutpB(BASE+0x10+2*channel,0x00);
        rl32eOutpB(BASE+0x11+2*channel,0x08);    // PR -> CNTR
        rl32eOutpB(BASE+0x11+2*channel,0x38);    // CMR
        rl32eOutpB(BASE+0x11+2*channel,0x41);    // IOR
        rl32eOutpB(BASE+0x11+2*channel,0x65);    // IDR (index pulse polarity)
    }

#endif
                 
}

static void mdlOutputs(SimStruct *S, int_T tid)
{

#ifndef MATLAB_MEX_FILE

    int_T i, n, channel, irc;
    uint_T ad;
    real_T *y;
    short adParameters;
    real_T res;

	
    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        channel=(uint_T)mxGetPr(CHANNEL_ARG)[i]-1;
        y=ssGetOutputPortSignal(S,i);

        ad=BASE+0x10+2*channel;
        rl32eOutpB(ad+1,0x11);    // CNTR -> OL, reset BP
        irc=rl32eInpB(ad)&0xff;
        irc|=(rl32eInpB(ad)&0xff)<<8;
        irc|=(rl32eInpB(ad)&0xff)<<16;
        if (irc>8388607) irc-=16777216;

        y[0]=(real_T)irc; 
    }

#endif
        	        
}

static void mdlTerminate(SimStruct *S)
{   
}


#ifdef MATLAB_MEX_FILE  /* Is this file being compiled as a MEX-file? */
#include "simulink.c"   /* Mex glue */
#else
#include "cg_sfun.h"    /* Code generation glue */
#endif


