/* $Revision:  $ $Date:  $ */
/* ctrhsmf604.c - xPC Target, non-inlined S-function driver for counter section of Humusoft MF604 board  */
/* Copyright (c) 2001 by Humusoft, s.r.o. All Rights Reserved. */

#define 	S_FUNCTION_LEVEL 	2
#undef 		S_FUNCTION_NAME
#define 	S_FUNCTION_NAME 	ctrhsmf604

#include 	<stddef.h>
#include 	<stdlib.h>

#include 	"simstruc.h" 

#ifdef 		MATLAB_MEX_FILE
#include 	"mex.h"
#else
#include 	<windows.h>
#include 	"io_xpcimport.h"
#endif

#define inpTDP()      inp(BASE+0)
#define outpTDP(val)  outp(BASE+0,(val))
#define outpTCP(val)  outp(BASE+1,(val))


/* Input Arguments */
#define NUM_PARAMS             (7)
#define BASE_ADDRESS_ARG       (ssGetSFcnParam(S,0))
#define DEV_ARG                (ssGetSFcnParam(S,1))
#define CHANNEL_ARG            (ssGetSFcnParam(S,2))
#define RESET_ARG              (ssGetSFcnParam(S,3))
#define CLOCK_ARG              (ssGetSFcnParam(S,4))
#define GATE_ARG               (ssGetSFcnParam(S,5))
#define SAMPLE_TIME_PARAM      (ssGetSFcnParam(S,6))

/* Convert S Function Parameters to Varibles */

#define BASE                   ((uint_T) mxGetPr(BASE_ADDRESS_ARG)[0])
#define SAMPLE_TIME            ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[0])
#define SAMPLE_OFFSET          ((real_T) mxGetPr(SAMPLE_TIME_PARAM)[1])


static char_T msg[256];


/*====================*
 * S-function methods *
 *====================*/

static void mdlCheckParameters(SimStruct *S)
{
}

static void mdlInitializeSizes(SimStruct *S)
{
    uint_T i;

#ifndef MATLAB_MEX_FILE
#include "io_xpcimport.c"
#endif


    ssSetNumSFcnParams(S, NUM_PARAMS);
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S)) {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL) {
            return;
        }
    } else {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

	

    if (!ssSetNumInputPorts(S, 0)) return;

    if (!ssSetNumOutputPorts(S, mxGetNumberOfElements(CHANNEL_ARG))) return;

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        ssSetOutputPortWidth(S, i, 1);
    }

    ssSetNumSampleTimes(S, 1);

    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetSFcnParamNotTunable(S,0);
    ssSetSFcnParamNotTunable(S,1);
    ssSetSFcnParamNotTunable(S,2);
    ssSetSFcnParamNotTunable(S,3);
    ssSetSFcnParamNotTunable(S,4);
    ssSetSFcnParamNotTunable(S,5);
    ssSetSFcnParamNotTunable(S,6);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE | SS_OPTION_PLACE_ASAP);
	
}


 
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, SAMPLE_TIME);
    ssSetOffsetTime(S, 0, SAMPLE_OFFSET);
}

 

#define MDL_START
static void mdlStart(SimStruct *S)
{

#ifndef MATLAB_MEX_FILE

    uint_T i, channel, gate;
    char devName[20];

    strcpy(devName,"Humusoft MF604");

    outpTCP(0x17);                 // master mode register
    outpTDP(0x00);
    outpTDP(0x80);                 // BCD prescaler

    gate=mxGetPr(GATE_ARG)[0];
    if (gate>0) gate+=3;

    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        channel=(uint_T)mxGetPr(CHANNEL_ARG)[i]-1;

        outpTCP(channel+1);                     // select counter mode register
        outpTDP(0xA8);
        outpTDP((1+channel) | ((uint_T)mxGetPr(CLOCK_ARG)[0] << 4) | (gate << 5));   // count up, repetition
        outpTCP(0x60 | (1 << channel));         // load & arm counter
    }

#endif
                 
}

static void mdlOutputs(SimStruct *S, int_T tid)
{

#ifndef MATLAB_MEX_FILE

    int_T i, channel, mask;
    real_T *y;
    real_T res;

	
    for (i=0;i<mxGetNumberOfElements(CHANNEL_ARG);i++) {
        channel=(uint_T)mxGetPr(CHANNEL_ARG)[i]-1;
        y=ssGetOutputPortSignal(S,i);

        mask=1<<channel;                     // mask for counters

        if(mxGetPr(CHANNEL_ARG)[0]==0)
          {
            outpTCP(0xA0 | mask);            // save counters
          }
        else
          {
            outpTCP(0x80 | mask);            // disarm & save counters
            outpTCP(0x60 | mask);            // load & arm counters
          }

        y[0]=(real_T)(inpTDP()+(inpTDP()<<8)); 
    }

#endif
        	        
}

static void mdlTerminate(SimStruct *S)
{   
}


#ifdef MATLAB_MEX_FILE  /* Is this file being compiled as a MEX-file? */
#include "simulink.c"   /* Mex glue */
#else
#include "cg_sfun.h"    /* Code generation glue */
#endif


